% RG Index Algorithm

% Published in: Arabian Journal of Geosciences, vol. 9, no. 1, p. 45, Jan. 2016.
% Title: "Quality assessment of pan-sharpening methods in high-resolution
% satellite images using radiometric and geometric index"
% Authors: M. Hasanlou and M. R. Saradjian

% Overall explanation.
% This code focuses on quality assessment of pansharpening of multispectral (MS)
% images with high-resolution panchromatic (Pan) images. Since most existing quality
% assessments take the entire image into account simultaneously and generate some uncertainties,
% a novel and rather objective quality index has been proposed for image fusion.
% The index is comprised of geometric and radiometric parts. Both geometric and radiometric measurements are calculated
% using morphological algorithm applied on an edge image to create a mask which is used to separate highfrequency regions
% from low-frequency ones.The accuracy assessment is made using common existing criteria on geometric and radiometric segments,
% and then a weighted sum is calculated to generate radiometric and geometric index (RG index).

clc
clear all

% Load PAN and MS images 
nDataset = 1;

pathData_MS  = [pwd '\Data-Set' num2str(nDataset) '\MS.tif'];
pathData_PAN = [pwd '\Data-Set' num2str(nDataset) '\PAN.tif'];

MS_Image  = imread(pathData_MS);
PAN_Image = imread(pathData_PAN);

FusionMethod = {'IHS','IHS*','PCA','GS','BT','EHL','HPM','SWT'};
nFusionMethod = 1;    % Pansharpening method

[FusionImage,EdgeMapPAN,Quality_Indices] = RGIndex(PAN_Image,MS_Image,nFusionMethod);

figure,
subplot 221
J(:,:,1) = imadjust(MS_Image(:,:,3));
J(:,:,2) = imadjust(MS_Image(:,:,2));
J(:,:,3) = imadjust(MS_Image(:,:,1));
imagesc(J);axis off, title('MultiSpectral Image','Color','b')

subplot 222
J = imadjust(PAN_Image);
imagesc(J);axis off,colormap gray,title('PAN Image','Color','b')

subplot 223
FusionImage16 =  uint16(FusionImage);
J(:,:,1) = imadjust(FusionImage16(:,:,1));
J(:,:,2) = imadjust(FusionImage16(:,:,2));
J(:,:,3) = imadjust(FusionImage16(:,:,3));
imagesc(J);axis off,title(['Fused Image using ' FusionMethod(nFusionMethod)],'Color','b')

subplot 224
J = EdgeMapPAN;
imagesc(J);axis off,title('Edge Map','Color','b')

str = sprintf('Method:	Radiometric\tGeometric\tAvarage  \tOld Index');
disp(str)
disp('-----------------------------------------------------');
str = sprintf('SAM   : %f\t%f\t%f\t%f',Quality_Indices.SAM(1),Quality_Indices.SAM(2),Quality_Indices.SAM(3),Quality_Indices.SAM(4));
disp(str)
str = sprintf('SSIM  : %f\t%f\t%f\t%f',Quality_Indices.SSIM(1),Quality_Indices.SSIM(2),Quality_Indices.SSIM(3),Quality_Indices.SSIM(4));
disp(str)
str = sprintf('CC    : %f\t%f\t%f\t%f',Quality_Indices.Correlation(1),Quality_Indices.Correlation(2),Quality_Indices.Correlation(3),Quality_Indices.Correlation(4));
disp(str)
str = sprintf('UQI   : %f\t%f\t%f\t%f',Quality_Indices.UQI(1),Quality_Indices.UQI(2),Quality_Indices.UQI(3),Quality_Indices.UQI(4));
disp(str)
str = sprintf('RMSE  : %f\t%f\t%f\t%f',Quality_Indices.RMSE(1),Quality_Indices.RMSE(2),Quality_Indices.RMSE(3),Quality_Indices.RMSE(4));
disp(str)
str = sprintf('PSNR  : %f\t%f\t%f\t%f',Quality_Indices.PSNR(1),Quality_Indices.PSNR(2),Quality_Indices.PSNR(3),Quality_Indices.PSNR(4));
disp(str)
str = sprintf('SDD   : %f\t%f\t%f\t%f',Quality_Indices.SDD(1),Quality_Indices.SDD(2),Quality_Indices.SDD(3),Quality_Indices.SDD(4));
disp(str)
